
#include "atom.h"

// Constructor
//
Atom::Atom(ElementBuilder* eb, char* symbol, float x, float y, float z) {

  //printf("=>atom()\n");

  element = eb->getElement(symbol);
  coord = new Coordinate3D(x,y,z);

  //printf("<=atom()\n");

  return;
}


Atom::Atom(ElementBuilder* eb, char* symbol, Coordinate3D* c) {

  //printf("=>atom()\n");

  element = eb->getElement(symbol);
  coord = new Coordinate3D(c->getX(),c->getY(),c->getZ());

  //printf("<=atom()\n");

  return;
}


Atom::Atom(ElementBuilder* eb, char* symbol, float x, float y, float z, float ch) :
  charge(ch) {

  //printf("=>atom()\n");

  element = eb->getElement(symbol);
  coord = new Coordinate3D(x,y,z);

  //printf("<=atom()\n");

  return;
}


Atom::Atom(ElementBuilder* eb, char* symbol, Coordinate3D* c, float ch) :
  charge(ch) {

  //printf("=>atom()\n");

  element = eb->getElement(symbol);
  coord = new Coordinate3D(c->getX(),c->getY(),c->getZ());

  //printf("<=atom()\n");

  return;
}


// Destructor
//
Atom::~Atom() {

  delete element;
  delete coord;

  return;
}


// getCoordinate3D
//
Coordinate3D* Atom::getCoordinate3D() {

  return coord;
}


// getCharge
//
float Atom::getCharge() {

  return charge;
}


// isWithinVdwRadius
//   Whether the given coordinate is within the Van der
//   Waals radius of the atom or not
int Atom::isWithinVdwRadius(Coordinate3D* c) {

  //printf("=>Atom::isWithinVdwRadius()\n");

  float distance = coord->getDistanceTo(c);
  //printf("  element: %s\n", element->getSymbol());

  // XXX - CHANGED FOR NULLSPACE CALCULATION
  //  if (distance <= element->getVdwRadius()) {
  if (distance <= (element->getVdwRadius() + 1.2)) {
    //printf("<=Atom::isWithinVdwRadius(): 1\n");
    return 1;
  }

  //printf("<=Atom::isWithinVdwRadius(): 0\n");
  return 0;
}


/* XXX - MAKE NEW VERSION THAT TAKES PADDING AS ARGUMENT

// isWithinVdwRadius
//   Whether the given coordinate is within the Van der
//   Waals radius of the atom or not
int Atom::isWithinVdwRadius(Coordinate3D* c) {

  //printf("=>Atom::isWithinVdwRadius()\n");

  float distance = coord->getDistanceTo(c);
  //printf("  Hey\n");
  //printf("  element: %s\n", element->getSymbol());

  // XXX - CHANGED FOR NULLSPACE CALCULATION
  //  if (distance <= element->getVdwRadius()) {
  if (distance <= (element->getVdwRadius() + 1.0)) {
    //printf("<=Atom::isWithinVdwRadius(): 1\n");
    return 1;
  }

  //printf("<=Atom::isWithinVdwRadius(): 0\n");
  return 0;
}
*/
