
#include "baseVertexNodeTree.h"


/**
 * Constructor.
 */
BaseVertexNodeTree::BaseVertexNodeTree(BaseVertexNode* v) :
  vertex(v), child1(0), child2(0) {

  return;
}


/**
 * Destructor.  Not responsible for deleting the BaseVertexNodes.
 */
BaseVertexNodeTree::~BaseVertexNodeTree() {

  if (child1 != 0) {
    delete child1;
  }
  if (child2 != 0) {
    delete child2;
  }

  return;
}


/**
 * Add a node to the tree.
 * @param node BaseVertexNode*: node to add.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNodeTree::insert(BaseVertexNode* node) {

  Coordinate3D* coord1 = node->getCoordinate3D();
  Coordinate3D* coord2 = vertex->getCoordinate3D();

  if (coord1->isLessThan(coord2)) {
    if (child1 != 0) {
      return child1->insert(node);
    }
    else {
      child1 = new BaseVertexNodeTree(node);
      return 1;
    }
  }
  
  if (child2 != 0) {
    return child2->insert(node);
  }
  else {
    child2 = new BaseVertexNodeTree(node);
    return 1;
  }

  // XXX DEBUG - NEED TO HANDLE CASE WHERE COORDINATE3D IS SAME AS EXISTING ONE

  return 0;
}


/**
 * Retrieve an object stored in the tree.
 * @param coord Coordinate3D*: lookup key.
 * @return BaseVertexNode*: retrieved object.
 */
BaseVertexNode* BaseVertexNodeTree::retrieve(Coordinate3D* coord1) {

  Coordinate3D* coord2 = vertex->getCoordinate3D();

  if (coord1->isLessThan(coord2)) {
    if (child1 != 0) {
      return child1->retrieve(coord1);
    }
    else {
      return 0;
    }
  }
  
  if (coord1->isGreaterThan(coord2)) {
    if (child2 != 0) {
      return child2->retrieve(coord1);
    }
    else {
      return 0;
    }
  }

  if (coord1->equals(coord2)) {
    return vertex;
  }

  return 0;
}

