
#include "connectionMatrix.h"


/********************
 *  ConnectionNode  *
 ********************/

// Constructor
//
ConnectionNode::ConnectionNode(int i, int val) :
  index(i), value(val) {

  next = 0;

  return;
}


// Destructor
//
ConnectionNode::~ConnectionNode() {

  if (next != 0) {
    delete next;
  }

  return;
}


ConnectionNode* ConnectionNode::getNext() {
  
  return next;
}


// getIndex
//
int ConnectionNode::getIndex() {

  return index;
}


// getValue
//
int ConnectionNode::getValue() {

  return value;
}


// setValue
int ConnectionNode::setValue(int val) {

  value = val;
  return 1;
}


/*
// setBefore
//
int ConnectionNode::setBefore(ConnectionNode* b) {

if (b != 0) {
before = b;
return 1;
}

return 0;
}
*/


// setAfter
//
int ConnectionNode::setNext(ConnectionNode* n) {

  next = n;
  return 1;
}



/********************
 *  ConnectionList  *
 ********************/

// Constructor
//
ConnectionList::ConnectionList() :
  head(0) {

  return;
}


// Destructor
//
ConnectionList::~ConnectionList() {

  delete head;

  return;
}


// getValue
//   Get the value of node i (0 if it's not there)
int ConnectionList::getValue(int i) {

  ConnectionNode* tempNode = head;

  while (tempNode != 0) {
    if (tempNode->getIndex() == i) {
      return tempNode->getValue();
    }
    tempNode = tempNode->getNext();
  }

  return -1;
}


// setValue
//   Set the value of node i
int ConnectionList::setValue(int i, int value) {

  if (value < 0) {
    remove(i);
    return 1;
  }

  if (value >= 0) {
    ConnectionNode* tempNode = head;
    while (tempNode != 0) {
      if (tempNode->getIndex() == i) {
	tempNode->setValue(value);
	return 1;
      }
      tempNode = tempNode->getNext();
    }
    add(i, value);
    return 1;
  }
  
  // Don't set negative values
  return 0;
}


// incrementValues
//   Increment the values of all the nodes in the list
int ConnectionList::incrementValues() {

  ConnectionNode* tempNode1 = head;
  while (tempNode1 != 0) {
    /*
      if (tempNode1->getValue() <= 1) {
      ConnectionNode* tempNode2 = tempNode1;
      tempNode1 = tempNode1->getNext();
      remove(tempNode2->getIndex());
      }
      else {
    */
    tempNode1->setValue(tempNode1->getValue() + 1);
    tempNode1 = tempNode1->getNext();
    //}
  }

  return 1;
}


// removeIfGreaterThan
//   Remove elements with values greater than limit
int ConnectionList::removeIfGreaterThan(float limit) {

  //printf("  =>ConnectionList::removeIfGreaterThan()\n");

  while ( head != 0 &&
	  (float)head->getValue() > limit ) {
    remove(head->getIndex());
  }

  if (head != 0 && head->getNext() != 0) {
    ConnectionNode* tempNode1 = head->getNext();
    ConnectionNode* tempNode2 = head;
    while (tempNode1 != 0) {
      if ((float)tempNode1->getValue() > limit) {
	remove(tempNode1->getIndex());
	tempNode1 = tempNode2->getNext();
      }
      else {
	tempNode2 = tempNode1;
	tempNode1 = tempNode1->getNext();
      }
    }
  }
  
  //printf("  <=ConnectionList::removeIfGreaterThan()\n");

  return 1;
}


// add
//   Add a node to the list
int ConnectionList::add(int i, int value) {

  // No elements in list so create first element
  if (head == 0) {
    head = new ConnectionNode(i, value);
    return 1;
  }

  // Index is < head index so insert at head
  if (head->getIndex() > i) {
    ConnectionNode* newNode = new ConnectionNode(i, value);
    newNode->setNext(head);
    head = newNode;
    return 1;
  }

  // Only a head so add to end of head
  ConnectionNode* tempNode1 = head->getNext();

  if (tempNode1 == 0) {
    if (head->getIndex() == i) {
      printf("Error: ConnectionList add() - trying to clobber an element\n");
      return 0;
    }
    if (head->getIndex() < i) {
      ConnectionNode* newNode = new ConnectionNode(i, value);
      head->setNext(newNode);
      return 1;
    }
    else {
      printf("Error: ConnectionList add() - Something screwey is going on\n");
      return 0;
    }
  }

  // Full on list so insert where needed
  ConnectionNode* tempNode2 = head;

  while (tempNode1 != 0) {
    if (tempNode1->getIndex() == i) {
      printf("Error: ConnectionList add() - trying to clobber an element\n");
      return 0;
    }
    if (tempNode1->getIndex() > i) {
      ConnectionNode* newNode = new ConnectionNode(i, value);
      newNode->setNext(tempNode1);
      tempNode2->setNext(newNode);
      return 1;
    }
    tempNode2 = tempNode1;
    tempNode1 = tempNode1->getNext();
    //printf("  Hey\n");
  }

  // index > all elements in list so add at end
  ConnectionNode* newNode = new ConnectionNode(i, value);
  tempNode2->setNext(newNode);

  return 1;
}


// remove
//   Remove a node from the list
int ConnectionList::remove(int i) {

  // Nothing in list
  if (head == 0) {
    //printf("Error: ConnectionList remove() - no elements in list\n");
    return 1;
  }

  // i == head so remove head
  if (head->getIndex() == i) {
    ConnectionNode* tempNode1 = head->getNext();
    if (tempNode1 == 0) {
      delete head;
      head = 0;
      return 1;
    }
    ConnectionNode* tempNode2 = head;
    tempNode2->setNext(0);
    delete tempNode2;
    head = tempNode1;
    return 1;
  }

  // i is in body of list
  ConnectionNode* tempNode1 = head->getNext();
  ConnectionNode* tempNode2 = head;
  while (tempNode1->getNext() != 0) {
    if (tempNode1->getIndex() == i) {
      ConnectionNode* tempNode3 = tempNode1;
      tempNode1 = tempNode1->getNext();
      tempNode2->setNext(tempNode1);
      tempNode3->setNext(0);
      delete tempNode3;
      return 1;
    }
    tempNode2 = tempNode1;
    tempNode1 = tempNode1->getNext();
  }

  // i is at end of list
  if (tempNode1->getIndex() == i) {
    tempNode2->setNext(0);
    delete tempNode1;
    return 1;
  }

  // if i isn't in list, do nothing

  return 1;
}



/**********************
 *  ConnectionMatrix  *
 **********************/

// Constructor
//
ConnectionMatrix::ConnectionMatrix(int l) :
  listLength(l) {

  conList = new ConnectionList*[listLength];

  for (int i=0; i<listLength; i++) {
    conList[i] = new ConnectionList();
  }

  return;
}


// Destructor
//
ConnectionMatrix::~ConnectionMatrix() {

  for (int i=0; i<listLength; i++) {
    delete conList[i];
  }

  delete conList;

  return;
}


// getValue
//
int ConnectionMatrix::getValue(int i, int j) {

  if (i < j) {
    return conList[i]->getValue(j);
  }

  if (i > j) {
    return conList[j]->getValue(i);
  }

  // Don't set values for i==j because an element can't be
  //   connected to itself
  return -1;
}


// setValue
//
int ConnectionMatrix::setValue(int i, int j, int value) {
  
  if (i < j) {
    return conList[i]->setValue(j, value);
  }

  if (i > j) {
    return conList[j]->setValue(i, value);
  }

  // Don't set values for i==j because an element can't be
  //   connected to itself
  return 1;
}


// incrementValues
//   Increment values of all the nonnegative matrix elements
int ConnectionMatrix::incrementValues() {

  for (int i=0; i<listLength; i++) {
    conList[i]->incrementValues();
  }
  return 1;
}


// removeIfGreaterThan
//   Remove elements with values greater than limit
int ConnectionMatrix::removeIfGreaterThan(float limit) {

  //printf("=>ConnectionMatrix::removeIfGreaterThan()\n");

  for (int i=0; i<listLength; i++) {
    conList[i]->removeIfGreaterThan(limit);
  }

  //printf("<=ConnectionMatrix::removeIfGreaterThan()\n");

  return 1;
}
