
#ifndef COORDINATE3D_H
#define COORDINATE3D_H

#include <math.h>
#include <stdio.h>

class Coordinate3D {
  
 public:
  Coordinate3D(float a, float b, float c);
  ~Coordinate3D();

  int equals(Coordinate3D* coord);
  int isGreaterThan(Coordinate3D* coord);
  int isLessThan(Coordinate3D* coord);
  float getX();
  float getY();
  float getZ();
  void setX(float a);
  void setY(float b);
  void setZ(float c);
  float getDistanceTo(Coordinate3D* coord1);
  Coordinate3D* clone();
  Coordinate3D* plus(Coordinate3D* coord1);
  Coordinate3D* minus(Coordinate3D* coord1);
  float dot(Coordinate3D* coord1);
  Coordinate3D* cross(Coordinate3D* coord1);

  /**
   * Rotate the coordinate a radians about the X axis.
   * @param a float: angle of rotation in radians.
   * @return int: 0 on fail, 1 on success.
   */
  int rotateX(float a);

  /**
   * Rotate the coordinate a radians about the Y axis.
   * @param a float: angle of rotation in radians.
   * @return int: 0 on fail, 1 on success.
   */
  int rotateY(float a);

  /**
   * Rotate the coordinate a radians about the Z axis.
   * @param a float: angle of rotation in radians.
   * @return int: 0 on fail, 1 on success.
   */
  int rotateZ(float a);

 private:
  float x;
  float y;
  float z;
  
};

#endif
