
#include "neuronList2.h"


// Constructor
//
NeuronNode::NeuronNode(Coordinate3D* c, int i, float d) :
  next(0), prev(0), coord(c), index(i), distance(d) {
  //printf("NeuronNode::NeuronNode()\n");
  return;
}


// Destructor
//
NeuronNode::~NeuronNode() {

  if (next != 0) {
    delete next;
  }

  // DON'T DELETE COORDINATE3D

  return;
}


// getNext
//
NeuronNode* NeuronNode::getNext() {

  return next;
}


// getPrev
//
NeuronNode* NeuronNode::getPrev() {

  return prev;
}


// getCoord
//
Coordinate3D* NeuronNode::getCoord() {

  return coord;
}


// getIndex
//
int NeuronNode::getIndex() {

  return index;
}


// getDistance
//
float NeuronNode::getDistance() {

  return distance;
}


// setNext
//
int NeuronNode::setNext(NeuronNode* n) {

  next = n;
  return 1;
}


// setPrev
//
int NeuronNode::setPrev(NeuronNode* p) {

  prev = p;
  return 1;
}


// isLessThan
//
int NeuronNode::isLessThan(NeuronNode* n) {

  if (distance < n->getDistance()) {
    return 1;
  }

  return 0;
}


// isGreaterThan
//
int NeuronNode::isGreaterThan(NeuronNode* n) {

  if (distance > n->getDistance()) {
    return 1;
  }

  return 0;
}




// Constructor
//
NeuronList::NeuronList(Coordinate3D** coords, int coordCount, Coordinate3D* f) :
  head(0), current(0), focus(f), neuronCount(coordCount)  {

  // Load NeuronList with coordinates
  for (int i=0; i<neuronCount; i++) {
    addNeuronNode(coords[i], i, coords[i]->getDistanceTo(focus));
    //printf("<=NeuronList::addNeuronNode()\n");
  }

  checkOrder();

  return;
}


// Destructor
//
NeuronList::~NeuronList() {

  if (head != 0) {
    delete head;
  }

  // XXX - MAYBE DELETE FOCUS
  delete focus;

  return;
}


// addNeuronNode
//
int NeuronList::addNeuronNode(Coordinate3D* coord, int index, float distance) {

  //printf("=>NeuronList::addNeuronNode()\n");
  //printf("   index: %d, distance: %3.2f\n",index,distance);

  NeuronNode* node = new NeuronNode(coord, index, distance);
  
  if (head == 0) {
    head = node;
    current = node;
    //printf("   head == 0\n");
    return 1;
  }

  //printf("   Hey1\n");
  NeuronNode* tempNode1 = current;
  NeuronNode* tempNode2 = current;
  //printf("   Hey2\n");

  // Move down the list to the right
  if (distance >= tempNode2->distance) {
    //printf("   in if 1\n");
    tempNode1 = tempNode1->next;
    while ( tempNode1 != 0 &&
	    distance > tempNode1->distance ) {
      tempNode2 = tempNode1;
      tempNode1 = tempNode1->next;
    }
    if (tempNode1 == 0) {
      tempNode2->setNext(node);
      node->setPrev(tempNode2);
      //printf("   return 1\n");
      return 1;
    }
    tempNode2->setNext(node);
    node->setPrev(tempNode2);
    tempNode1->setPrev(node);
    node->setNext(tempNode1);
    //printf("   return 2\n");
    return 1;
  }

  // Move down the list to the left
  if (distance < tempNode2->distance) {
    //printf("   in if 2\n");
    tempNode1 = tempNode1->prev;
    while ( tempNode1 != 0 &&
	    distance < tempNode1->distance ) {
      tempNode2 = tempNode1;
      tempNode1 = tempNode1->prev;
    }
    if (tempNode1 == 0) {
      tempNode2->setPrev(node);
      node->setNext(tempNode2);
      head = node;
      //printf("   return 3\n");
      return 1;
    }
    tempNode2->setPrev(node);
    node->setNext(tempNode2);
    tempNode1->setNext(node);
    node->setPrev(tempNode1);
    //printf("   return 4\n");
    return 1;
  }

  //printf("   return 5\n");
  return 0;
}


// getFirstIndex
//
int NeuronList::getFirstIndex() {

  if (head != 0) {
    Coordinate3D* n = head->getCoord();
    //printf("  1st %d: (%f,%f,%f)\n",head->getIndex(),n->getX(),n->getY(),n->getZ());
    return head->getIndex();
  }

  return 0;
}


// getSecondIndex
//
int NeuronList::getSecondIndex() {

  if (head != 0 && head->getNext() != 0) {
    Coordinate3D* n = head->getNext()->getCoord();
    //printf("  2nd %d: (%f,%f,%f)\n",head->getNext()->getIndex(),n->getX(),n->getY(),n->getZ());
    return head->getNext()->getIndex();
  }

  return 0;
}


// contract
//   Move all the coordinates towards the focus
int NeuronList::contract(float stepSize, float lambda) {

  printf("\n    stepSize: %f, lambda: %f\n",stepSize,lambda);

  if (stepSize == 0 || lambda == 0) {
    // Don't contract
    return 1;
  }

  NeuronNode* tempNode = head;
  Coordinate3D* coord = 0;
  int i=0;
  float coeff = 0.0;
  float x = 0.0;
  float y = 0.0;
  float z = 0.0;
  while (tempNode != 0) {
    printf("%d:%f ",tempNode->getIndex(),tempNode->getDistance());
    if (i%10 == 0) {
      printf("\n");
    }
    coord = tempNode->getCoord();
    /*
    x = focus->getX() - coord->getX();
    y = focus->getY() - coord->getY();
    z = focus->getZ() - coord->getZ();
    */
    // NEW {
    x = focus->coords[0] - coord->coords[0];
    y = focus->coords[1] - coord->coords[1];
    z = focus->coords[2] - coord->coords[2];
    // } NEW
    coeff = stepSize * exp( (-1 * i) / lambda );
    //coeff = stepSize * exp( i / lambda );   // NEW
    x *= coeff;
    y *= coeff;
    z *= coeff;
    /*
    coord->setX( coord->getX() + x );
    coord->setY( coord->getY() + y );
    coord->setZ( coord->getZ() + z );
    */
    // NEW {
    coord->coords[0] += x;
    coord->coords[1] += y;
    coord->coords[2] += z;
    // } NEW
    tempNode = tempNode->next;
    i++;
    //i--;   // NEW
  }

  return 1;
}


// checkOrder
//   DEBUG METHOD
int NeuronList::checkOrder() {

  int c = 0;
  int i = 0;
  NeuronNode* tempNode = head;
  while (tempNode->next != 0) {
    if (tempNode->isGreaterThan(tempNode->next) == 1) {
      printf("   nodes[%d]->distance = %f; nodes[%d]->distance = %f\n",i,tempNode->distance,i+1,tempNode->next->distance);
      c++;
    }
    i++;
    tempNode = tempNode->next;
  }

  //printf("   Order Errors: %d\n",c);

  return 1;
}
