
#include "tessellationScriptReader.h"


/**
 * Constructor.
 */
TessellationScriptReader::TessellationScriptReader() :
  filename(0), path(0), fullName(0){

  return;
}


/**
 * Destructor.
 */
TessellationScriptReader::~TessellationScriptReader() {

  delete filename;
  delete path;
  delete fullName;

  return;
}


/**
 */
int TessellationScriptReader::setFilename(char* fn) {

  if (filename != 0) {
    delete filename;
    filename = 0;
  }

  if (fullName != 0) {
    delete fullName;
    fullName = 0;
  }

  int len = strlen(fn);
  char* tempName = new char[len+5];  // len+1 is normal (+4) for suffix
  strncpy(tempName,fn,len);
  tempName[len] = '\0';

  filename = new char[len+1];
  strncpy(filename,fn,len);
  filename[len] = '\0';

  // Given name works
  if (checkFullName() == 1) {
    //printf("\n      %s works\n\n",filename);
    //setNameFromFilename();
    //printf("<=PdbReader::setFilename\n");
    return 1;
  }
  //printf("   %s doesn't work\n",filename);

  delete filename;
  filename = new char[len+5];
  strncpy(filename,fn,len);
  filename[len] = '\0';
  strncat(filename,".tcl",4);

  // Name + ".pdb" works
  if (checkFullName() == 1) {
    //printf("\n      %s works\n\n",filename);
    // Always setNameFromFilename, people can change the name later
    //setNameFromFilename();
    //printf("<=PdbReader::setFilename\n");
    return 1;
  }
  //printf("   %s doesn't work\n",filename);
  
  delete filename;
  filename = 0;

  //printf("<=setFilename: 0\n");

  return 0;
}


/**
 */
int TessellationScriptReader::setPath(char* p) {

  //printf("=>setPath\n");
  //printf("  p: %s\n",p);
  int len = strlen(p);
  if (path != 0)  delete path;
  path = new char[len+1];
  strncpy(path,p,len);
  path[len] = '\0';
  //printf("<=setPath\n");

  return 1;
}


/**
 */
BaseGraph* TessellationScriptReader::getBaseGraph() {

  int vertexCount = getVertexCount();
  BaseGraph* graph = new BaseGraph(vertexCount);

  // Get vertices
  getVertices(graph, vertexCount);

  // Get edges
  getEdges(graph);

  return graph;
}


/**
 */
int TessellationScriptReader::getVertices(BaseGraph* graph, int vertexCount) {
  
  FILE* infile = fopen(fullName,"r");

  if (infile == NULL) {
    return 0;
  }

  char* tempStr = new char[1024];
  int i = 0;
  Coordinate3D* tempCoord = 0;

  fgets(tempStr,1023,infile);
  while (!feof(infile) && i < vertexCount) {
    //printf("%s\n",tempStr);
    if ( strncmp(tempStr,"graphics $mol point", 19) == 0 ) {
      tempCoord = getCoord3DFromString(tempStr+19);
      graph->addVertex(tempCoord);
      i++;
    }
    if ( strncmp(tempStr,"graphics $mol sphere",20) == 0 ) {
      tempCoord = getCoord3DFromString(tempStr+20);
      graph->addVertex(tempCoord);
      i++;
    }
    fgets(tempStr,1023,infile);
  }
  fclose(infile);

  return 1;
}


/**
 */
int TessellationScriptReader::getEdges(BaseGraph* graph) {
  
  FILE* infile = fopen(fullName,"r");

  if (infile == NULL) {
    return 0;
  }

  char* tempStr = new char[1024];
  int i = 0;
  EdgeCoords* edge = 0;

  fgets(tempStr,1023,infile);
  while (!feof(infile)) {
    //printf("%s\n",tempStr);
    if ( strncmp(tempStr,"graphics $mol line", 18) == 0 ) {
      edge = getEdgeCoordsFromString(tempStr+18);
      graph->addEdge(edge->coord1,edge->coord2);
      i++;
    }
    if ( strncmp(tempStr,"graphics $mol cylinder",22) == 0 ) {
      edge = getEdgeCoordsFromString(tempStr+22);
      graph->addEdge(edge->coord1,edge->coord2);
      i++;
    }
    fgets(tempStr,1023,infile);
  }
  fclose(infile);

  return 1;
}


/**
 */
int TessellationScriptReader::getVertexCount() {

  FILE* infile = fopen(fullName,"r");

  if (infile == NULL) {
    return 0;
  }

  char* tempStr = new char[1024];

  int len = 0;

  fgets(tempStr,1023,infile);
  while (!feof(infile)) {
    //printf("%s\n",tempStr);
    if ( ( strncmp(tempStr,"graphics $mol point", 19) == 0 ) ||
	 ( strncmp(tempStr,"graphics $mol sphere",20) == 0 ) ) {
      len++;
    }
    fgets(tempStr,1023,infile);
  }
  fclose(infile);

  //printf("   len = %d\n",len);
  //printf("Exiting getVertexCount()\n");

  return len;
}


/**
 */
TessellationScriptReader::EdgeCoords* TessellationScriptReader::getEdgeCoordsFromString(char* edgeString) {

  int esOffset = 0;

  while (edgeString[esOffset] != '{') {
    esOffset++;
  }
  
  Coordinate3D* coord1 = getCoord3DFromString(edgeString+esOffset);

  esOffset++;
  while (edgeString[esOffset] != '{') {
    esOffset++;
  }

  Coordinate3D* coord2 = getCoord3DFromString(edgeString+esOffset);

  if (coord1 == 0 || coord2 == 0) {
    return 0;
  }

  return new EdgeCoords(coord1,coord2);
}


/**
 */
Coordinate3D* TessellationScriptReader::getCoord3DFromString(char* coordString) {

  float* coordArray = new float[3];
  char* tempStr = new char[16];
  int csOffset = 0;
  int tsOffset = 0;

  int i=0;
  for (; i<3; i++) {
    tempStr[0] = '\0';
    while ( coordString[csOffset] == '{' ||
	    coordString[csOffset] == ' ' ) {
      csOffset++;
    }
    while ( coordString[csOffset] != ' ' &&
	    coordString[csOffset] != '}' ) {
      tempStr[tsOffset] = coordString[csOffset];
      tsOffset++;
      csOffset++;
    }
    tempStr[tsOffset] = '\0';
    //printf("  tempStr: %s, charToFloat(tempStr): %f\n",tempStr,charToFloat(tempStr));
    coordArray[i] = charToFloat(tempStr);
    tsOffset = 0;
  }

  Coordinate3D* coord = new Coordinate3D(coordArray[0],coordArray[1],coordArray[2]);
  delete coordArray;
  delete tempStr;

  //printf("  coordString: %s, coord: {%f, %f, %f}\n",coordString,coord->getX(),coord->getY(),coord->getZ());

  return coord;
}


/**
 */
int TessellationScriptReader::checkFullName() {

  //printf("=>PdbReader::checkFullName\n");
  //printf("  path (%d): %s\n",strlen(path),path);
  //printf("  filename (%d): %s\n",strlen(filename),filename);

  if (filename == 0) {
    return 0;
  }

  if (fullName == 0) {
    if (path == 0) {
      //printf("   checking1: %s\n",filename);
      //FILE* infile = fopen(filename,"r");
      //if (infile == NULL) {
      //fclose(infile);
      //return 0;
      //}
      //fclose(infile);
      if (checkTessellationFile(filename) == 0) {
	return 0;
      }
      int filenameLen = strlen(filename);
      fullName = new char[filenameLen + 1];
      strncpy(fullName,filename,filenameLen);
      fullName[filenameLen] = '\0';
      //printf("   return1\n");
      //printf("<=PdbReader::checkFullName\n");
      return 1;
    }
    else {
      int pathLen = strlen(path);
      int filenameLen = strlen(filename);
      fullName = new char[pathLen + filenameLen + 1];
      strncpy(fullName,path,pathLen);
      fullName[pathLen] = '\0';
      strncat(fullName,filename,filenameLen);
      fullName[pathLen + filenameLen] = '\0';
      //printf("   checking2: %s\n",fullName);
      //FILE* infile = fopen(fullName,"r");
      //if (infile == NULL) {
	//fclose(infile);
	//delete fullName;
	//fullName = 0;
	//printf("<=PdbReader::checkFullName: 0\n");
	//return 0;
	//}
      //fclose(infile);
      if (checkTessellationFile(fullName) == 0) {
	delete fullName;
	fullName = 0;
	//printf("<=PdbReader::checkFullName: 0\n");
	return 0;
      }
      //printf("   return2\n");
      //printf("<=PdbReader::checkFullName\n");
      return 1;
    }
  }
  else {
    //printf("   checking3: %s\n",fullName);
    //FILE* infile = fopen(fullName,"r");
    //if (infile == NULL) {
      //fclose(infile);
      //delete fullName;
      //fullName = 0;
      //printf("<=PdbReader::checkFullName\n");
      //return 0;
      //}
    //fclose(infile);
    if (checkTessellationFile(fullName) == 0) {
      delete fullName;
      fullName = 0;
      //printf("<=PdbReader::checkFullName: 0\n");
      return 0;
    }
    //printf("   return3\n");
    //printf("<=PdbReader::checkFullName\n");
    return 1;
  }

  //printf("<=checkFullName: 0\n");
  return 0;
}


/**
 */
int TessellationScriptReader::checkTessellationFile(char* file) {

  FILE* infile = fopen(file,"r");
  if (infile == NULL) {
    fclose(infile);
    return 0;
  }
  
  char* tempString = new char[1024];
  int flag = 0;

  while (!feof(infile) && flag == 0) {
    fgets(tempString,1023,infile);
    if ( strncmp(tempString,"graphics $mol",13) == 0 ) {
      flag++;
    }
  }
  fclose(infile);

  delete tempString;

  if (flag == 0) {
    return 0;
  }

  return 1;
}

