
#include "tessellator.h"

// Constructor
//
Tessellator::Tessellator(AtomCluster* a, int nNum, float vertRad) :
  atoms(a), neuronNum(nNum), vertexRadius(vertRad),
  stepNum(0), addTracker(0), verticesAreStable(0) {

  connections = new ConnectionMatrix(neuronNum);
  coords = new Coordinate3D*[neuronNum];
  //ages = new int[neuronNum];
  for (int i=0; i<neuronNum; i++) {
    // Set coords to a bunch of random coordinates within
    //   atoms' minimum enclosing box
    coords[i] = atoms->getRandomBoxCoord3D();
    //ages[i] = 0;
  }

  // XXX - original is neuronNum * 200
  stepNumMax = neuronNum * 200;
  //stepNumMax = neuronNum * 4000;
  stepSizeI = 0.3;
  stepSizeF = 0.05;
  lambdaI = 0.2 * (float)neuronNum;
  lambdaF = 0.01;
  //lifetimeI = 1.0 * (float)neuronNum;
  lifetimeI = 0.1 * (float)neuronNum;

  // XXX - original is 2.0 * (float)neuronNum;
  //lifetimeF = 2.0 * (float)neuronNum;
  lifetimeF = 2.0 * (float)neuronNum;

  hebbian = 1;

  return;
}


// Constructor
//
Tessellator::Tessellator(AtomCluster* a, int nNum, float vertRad, Coordinate3D** neurons, ConnectionMatrix* conMat) :
  atoms(a), neuronNum(nNum), vertexRadius(vertRad),
  coords(neurons), connections(conMat),
  stepNum(0), addTracker(0), verticesAreStable(0) {
  
  // XXX - check to make sure coords and connections both have neuronNum elements

  //stepNumMax = neuronNum * 200;

  // XXX
  // XXX - MAJOR CHANGE TO STEPNUMMAX CAUSING TESSELLATION TO SLOW DOWN!!!!!!!
  // XXX
  //stepNumMax = neuronNum * 300;
  stepNumMax = neuronNum * 200;
  stepSizeI = 0.3;
  stepSizeF = 0.05;
  lambdaI = 0.2 * (float)neuronNum;
  lambdaF = 0.01;
  lifetimeI = 0.1 * (float)neuronNum;
  lifetimeF = 2.0 * (float)neuronNum;

  hebbian = 1;

  return;
}


// Destructor
//
Tessellator::~Tessellator() {

  //delete atoms;
  delete connections;
  for (int i=0; i<neuronNum; i++) {
    delete coords[i];
  }

  delete coords;
  //delete neuronList;

  return;
}


// setStepNum
//   Set stepNum
int Tessellator::setStepNum(int i) {

  if (i < 0) {
    return 0;
  }

  stepNum = i;

  return 1;
}


// setHebbian
//   Allow or disallow tessellator to add and remove edges to the tessellation
int Tessellator::setHebbian(int i) {

  if (i == 0) {
    hebbian = 0;
    return 1;
  }

  hebbian = 1;

  return 1;
}


// stepTessellation
//
int Tessellator::stepTessellation() {

  //printf("=>Tessellator::stepTessellation()\n");

  // Define time-dependent step parameters
  float stepSizeParam = getTimeDependentParam(stepSizeI, stepSizeF);
  float lambdaParam   = getTimeDependentParam(lambdaI, lambdaF);
  float lifetimeParam = getTimeDependentParam(lifetimeI, lifetimeF);

  // Threshold of precision to stop calculation
  if (stepSizeParam <= 0.00001 || lambdaParam == 0.00001) {
    verticesAreStable = 1;
  }

  //if (stepSizeParam == 0 || lambdaParam == 0) {
  //  verticesAreStable = 1;
  //}

  // XXX - SWITCH BETWEEN REGULAR AND NULL SPACE
  //Coordinate3D* focus = atoms->getRandomVolumeCoord3D();
  Coordinate3D* focus = atoms->getRandomNullCoord3D();
  NeuronList* neuronList = new NeuronList(coords, neuronNum, focus);

  //if (hebbian == 1 && stepNum > 1000000) {
  if (hebbian == 1) {
    // Age all connections one step
    connections->incrementValues();

    // Create/refresh connection
    int firstIndex  = neuronList->getFirstIndex();
    int secondIndex = neuronList->getSecondIndex();
    if (connections->getValue(firstIndex,secondIndex) < 0) {
      addTracker++;
    }
    connections->setValue(firstIndex,secondIndex,0);
    //printf("  connections->setValue(%d,%d,0)\n",firstIndex,secondIndex);

    // Remove old connections
    connections->removeIfGreaterThan(lifetimeParam);
    //printf("  connections->removeIfGreaterThan(%3.2f)\n",lifetimeParam);
  }

  // Move all neurons towards focus
  neuronList->contract(stepSizeParam, lambdaParam);

  // XXX - DEBUG {
  //neuronList->getFirstIndex();
  //neuronList->getSecondIndex();
  // } XXX - DEBUG

  delete neuronList;
  //neuronList = 0;

  // XXX - DEBUG {
  
    //if (stepNum%1000 == 0) {
    //printf("   focus: (%f,%f,%f) ",focus->getX(),focus->getY(),focus->getZ());
    //}
  
  // } XXX - DEBUG

  delete focus;

  stepNum++;

  //printf("<=Tessellator::stepTessellation()\n");

  return 1;
}


int Tessellator::finalizeConnections() {

  //printf("=>Tessellator::finalizeConnections()\n");

  /*
    if (hebbian == 0) {
    printf("<=Tessellator::finalizeConnections()\n");
    return 1;
    }
  */
  
  float x = 0.0;
  float y = 0.0;
  float z = 0.0;
  float distance = 0.0;
  float cutoffDistance = getCutoffDistance();

  int i = 0;
  int j = 0;
  for (i=0; i<neuronNum; i++) {
    for (j=i+1; j<neuronNum; j++) {
      distance = coords[i]->getDistanceTo(coords[j]);
      //if (distance > cutoffDistance) {
      x = (coords[i]->getX() + coords[j]->getX()) / 2;
      y = (coords[i]->getY() + coords[j]->getY()) / 2;
      z = (coords[i]->getZ() + coords[j]->getZ()) / 2;
      Coordinate3D* focus = new Coordinate3D(x,y,z);
      NeuronList* neuronList = new NeuronList(coords, neuronNum, focus);
      
      // Age all connections one step
      // connections->incrementValues();
      
      // Create/refresh connection
      int firstIndex  = neuronList->getFirstIndex();
      int secondIndex = neuronList->getSecondIndex();
      if (connections->getValue(firstIndex,secondIndex) < 0) {
	addTracker++;
      }
      connections->setValue(firstIndex,secondIndex,1);
      
      //printf("[%d,%d] ",i,j);
      
      delete neuronList;
      delete focus;
      //}
    }
  }
  //printf("\n");
      
  //printf("<=Tessellator::finalizeConnections()\n");

  return 1;
}


int Tessellator::printTessellationScript(FILE* outfile) {

  if (neuronNum == 0) {
    return 0;
  }

  fprintf(outfile,"\nset mol top\n\n");

  for (int i=0; i<neuronNum; i++) {
    fprintf(outfile,
	    //	    "graphics $mol sphere {%3.2f %3.2f %3.2f} radius 0.6 resolution 12\n",
	    //"graphics $mol sphere {%3.2f %3.2f %3.2f} radius 0.2 resolution 10\n",
	    "graphics $mol point {%3.2f %3.2f %3.2f}\n",
	    coords[i]->getX(),
	    coords[i]->getY(),
	    coords[i]->getZ());
    for (int j=i+1; j<neuronNum; j++)
      if (connections->getValue(i,j) >= 0) {
	fprintf(outfile,
		//"graphics $mol cylinder {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f} radius 0.1 resolution 6\n",
		"graphics $mol line {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f}\n",
		coords[i]->getX(),
		coords[i]->getY(),
		coords[i]->getZ(),
		coords[j]->getX(),
		coords[j]->getY(),
		coords[j]->getZ());
      }
    fprintf(outfile,"\n");
  }

  return 1;
}


int Tessellator::getAddTracker() {

  return addTracker;
}


int Tessellator::resetAddTracker() {

  addTracker = 0;

  return 1;
}


int Tessellator::areVerticesStable() {
  return verticesAreStable;
}


float Tessellator::getVertexStability() {
  /*
    float exponent = (float)stepNum / (float)stepNumMax;
    float result = stepSizeF / stepSizeI;
    result = pow(result, exponent);
    result *= stepSizeI;
    
    printf("    stepNum: %d, stepNumMax: %d, stepSizeF: %f, stepSizeI: %f\n",stepNum,stepNumMax,stepSizeF,stepSizeI);
    printf("    exponent: %f, result: %f, calculated: %f\n",exponent,result,getTimeDependentParam(stepSizeI, stepSizeF));
  */
  return getTimeDependentParam(stepSizeI, stepSizeF);
}


float Tessellator::getTimeDependentParam(float initial, float final) {

  float exponent = (float)stepNum / (float)stepNumMax;
  float result = final / initial;
  result = pow(result, exponent);
  result *= initial;

  return result;
}


float Tessellator::getCutoffDistance() {

  float sumDistance = 0.0;
  float meanDistance = 0.0;
  int numDistances = 0;

  int i = 0;
  int j = 0;
  for (i=0; i<neuronNum; i++) {
    for (j=i+1; j<neuronNum; j++) {
      if (connections->getValue(i,j) >= 0) {
	sumDistance += coords[i]->getDistanceTo(coords[j]);
	numDistances++;
      }
    }
  }
  
  meanDistance = sumDistance / (float)numDistances;
  meanDistance *= 1.2;

  return meanDistance;
}
