
#ifndef TESSELLATOR_H
#define TESSELLATOR_H

#include "atomCluster.h"
#include "connectionMatrix.h"
#include "coordinate3D.h"
#include "neuronList.h"

#include <stdio.h>

class Tessellator {

 public:

  /** Constructor */
  Tessellator(AtomCluster* a, int nNum, float vertexRadius);

  /** Constructor */
  Tessellator(AtomCluster* a, int nNum, float vertexRadius, Coordinate3D** neurons, ConnectionMatrix* conMat);

  /** Destructor */
  ~Tessellator();

  /**
   *
   * @param int i:
   * @return int: 0 on fail, 1 on success.
   */
  int setStepNum(int i);

  /**
   *
   * @param int i: 
   * @return int: 0 on fail, 1 on success.
   */
  int setHebbian(int i);

  /**
   * Complete one step of the tessellation, moving vertices towards the focus and connecting the two closest vertices.
   * @return int: 0 on fail, 1 on success.
   */
  int stepTessellation();

  /**
   * Add last round of connections to complete the Delaunay tessellation.
   * @return int: 0 on fail, 1 on success.
   */
  int finalizeConnections();

  /**
   * 
   * @param FILE* outfile: file to print to.
   * @return int: 0 on fail, 1 on success.
   */
  int printTessellationScript(FILE* outfile);

  /**
   *
   * @return int: 0 on fail, 1 on success.
   */
  int getAddTracker();

  /**
   *
   * @return int: 0 on fail, 1 on success.
   */
  int resetAddTracker();

  /**
   * Whether simulated annealing has placed vertices stably or not.
   * @return int: 0 if vertices aren't stable, 1 if they are.
   */
  int areVerticesStable();

  /**
   * Debugging method.
   * @return float:
   */
  float getVertexStability();

 private:
  /**
   *
   * @param float initial: 
   * @param float final: 
   * @return float: time dependent parameter.
   */
  float getTimeDependentParam(float initial, float final);

  /**
   *
   * @return float: cutoff distance for making new connections.
   */
  float getCutoffDistance();

 private:
  AtomCluster* atoms;
  int neuronNum;
  Coordinate3D** coords;
  ConnectionMatrix* connections;
  //int* ages;
  // Created and deleted during each tessellation step
  //NeuronList* neuronList;
  //int maxAge;
  int   stepNum;
  int   stepNumMax;
  float stepSizeI;
  float stepSizeF;
  float lambdaI;
  float lambdaF;
  float lifetimeI;
  float lifetimeF;
  float vertexRadius;

  int   addTracker;   // Increments every time a connection is added
  int   verticesAreStable;   // Simulated annealing has completed the placement
  // of the vertices
  int   hebbian;    // 1: Add and remove edges from the tessellation, 0: don't

};

#endif
