
#ifndef TETRAHEDRONEDGETREE_H
#define TETRAHEDRONETREE_H

#include "coordinate3D.h"
#include "genericList.h"
//#include "triangleGraph.h"
#include "tetrahedronGraphNode.h"


struct IdPair {

  int x;
  int y;

};


struct TetrahedronEdge {

  Coordinate3D* coord1;
  Coordinate3D* coord2;
  Coordinate3D* coord3;

  /** Constructor */
  TetrahedronEdge(Coordinate3D* c1, Coordinate3D* c2, Coordinate3D* c3) {
    if (c1->isGreaterThan(c2)) {
      if (c1->isGreaterThan(c3)) {
	coord1 = c1;
	if (c2->isGreaterThan(c3)) {
	  coord2 = c2;
	  coord3 = c3;
	}
	else {
	  coord2 = c3;
	  coord3 = c2;
	}
      }
      else {
	coord1 = c3;
	coord2 = c1;
	coord3 = c2;
      }
    }
    else {
      if (c2->isGreaterThan(c3)) {
	coord1 = c2;
	if (c1->isGreaterThan(c3)) {
	  coord2 = c1;
	  coord3 = c3;
	}
	else {
	  coord2 = c3;
	  coord3 = c1;
	}
      }
      else {
	coord1 = c3;
	coord2 = c2;
	coord3 = c1;
      }
    }
    //return;
    //coord1 = c2;
    //coord2 = c1;
    return;
  }
  
  int equals(TetrahedronEdge* e) {
    
    if (coord1->equals(e->coord1) == 1) {
      if (coord2->equals(e->coord2) == 1) {
	if (coord3->equals(e->coord3) == 1) {
	  return 1;
	}
      }
    }

    return 0;
  }

  int isGreaterThan(TetrahedronEdge* e) {
    
    if (coord1->isGreaterThan(e->coord1) == 1) {
      return 1;
    }
    if (coord1->equals(e->coord1) == 1 &&
	coord2->isGreaterThan(e->coord2) == 1) {
      return 1;
    }
    if (coord1->equals(e->coord1) == 1 &&
	coord2->equals(e->coord2) == 1 &&
	coord3->isGreaterThan(e->coord3) == 1) {
      return 1;
    }

    return 0;
  }

  int isLessThan(TetrahedronEdge* e) {
    
    if (coord1->isLessThan(e->coord1) == 1) {
      return 1;
    }
    if (coord1->equals(e->coord1) == 1 &&
	coord2->isLessThan(e->coord2) == 1) {
      return 1;
    }
    if (coord1->equals(e->coord1) == 1 &&
	coord2->equals(e->coord2) == 1 &&
	coord3->isLessThan(e->coord3) == 1) {
      return 1;
    }

    return 0;
  }

};


/**
 * Binary tree arranged around edges defined by 3 coordinates. Each node corresponds to a specific face and links to a list of tetrahedra that all share the face. Helper class used by TetrahedronGraph to assign edges to the graph between tetrahedra that share a face.
 */
class TetrahedronEdgeTree {

 public:
  /** Constructor. */
  TetrahedronEdgeTree(TetrahedronEdge* e);

  TetrahedronEdgeTree(TetrahedronGraphNode* n);

  /** Destructor. */
  ~TetrahedronEdgeTree();


  /**
   * Add a node to the tree. Look at the 4 faces of the tetrahedron.  For each face, store a link to the tetrahedron at the corresponding node in the tree.
   * @param node TetrahedronGraphNode*: node to add.
   * @return int: 0 on fail, 1 on success.
   */
  int insert(TetrahedronGraphNode* node);

  /**
   * Add a node to the tree. Look at the 4 faces of the tetrahedron.  For each face, store a link to the tetrahedron at the corresponding node in the tree.
   * @param edge TetrahedronEdge*: face of tetrahedron.
   * @param triangle TetrahedronGraphNode*: tetrahedron to add.
   * @return int: 0 on fail, 1 on success.
   */
  int insert(TetrahedronEdge* e, TetrahedronGraphNode* tetrahedron);

  /**
   * Connect the nodes that correspond to tetrahedra sharing a face.
   * @return GenericList<IdPair*>*: list of TetrahedronNode id pairs for edge creation.
   */
   int createEdges(GenericList<IdPair*>* idList);

 private:
  TetrahedronEdge* edge;
  GenericList<TetrahedronGraphNode*>* edgeList;
  TetrahedronEdgeTree* child1;
  TetrahedronEdgeTree* child2;

};


#endif
