
#ifndef TRIANGLEEDGETREE_H
#define TRIANGLEEDGETREE_H

#include "coordinate3D.h"
#include "genericList.h"
//#include "triangleGraph.h"
#include "triangleGraphNode.h"


struct IntPair {

  int x;
  int y;

};


struct TriangleEdge {

  Coordinate3D* coord1;
  Coordinate3D* coord2;

  /** Constructor */
  TriangleEdge(Coordinate3D* c1, Coordinate3D* c2) {
    if (c1->isGreaterThan(c2)) {
      coord1 = c1;
      coord2 = c2;
      return;
    }
    coord1 = c2;
    coord2 = c1;
    return;
  }

  int equals(TriangleEdge* e) {
    
    if (coord1->equals(e->coord1) == 1) {
      if (coord2->equals(e->coord2) == 1) {
	return 1;
      }
    }

    return 0;
  }

  int isGreaterThan(TriangleEdge* e) {
    
    if (coord1->isGreaterThan(e->coord1) == 1) {
      return 1;
    }
    if (coord1->equals(e->coord1) == 1 &&
	coord2->isGreaterThan(e->coord2) == 1) {
      return 1;
    }

    return 0;
  }

  int isLessThan(TriangleEdge* e) {
    
    if (coord1->isLessThan(e->coord1) == 1) {
      return 1;
    }
    if (coord1->equals(e->coord1) == 1 &&
	coord2->isLessThan(e->coord2) == 1) {
      return 1;
    }

    return 0;
  }

};



/**
 * Binary tree arranged around edges defined by 2 coordinates. Each node corresponds to a specific edge and links to a list of triangles that all share the edge. Helper class used by TriangleGraph to assign edges to the graph between triangles that share a side.
 */
class TriangleEdgeTree {

 public:
  /** Constructor. */
  TriangleEdgeTree(TriangleEdge* e);

  TriangleEdgeTree(TriangleGraphNode* n);

  /** Destructor. */
  ~TriangleEdgeTree();


  /**
   * Add a node to the tree. Look at the 3 edges of the triangle.  For each edge, store a link to the triangle at the corresponding node in the tree.
   * @param node TriangleGraphNode*: node to add.
   * @return int: 0 on fail, 1 on success.
   */
  int insert(TriangleGraphNode* node);

  /**
   * Add a node to the tree. Look at the 3 edges of the triangle.  For each edge, store a link to the triangle at the corresponding node in the tree.
   * @param edge TriangleEdge*: edge of triangle.
   * @param triangle TriangleGraphNode*: triangle to add.
   * @return int: 0 on fail, 1 on success.
   */
  int insert(TriangleEdge* e, TriangleGraphNode* triangle);

  /**
   * Retrieve an object stored in the tree.
   * @param coord *: lookup key.
   * @return TriangleEdgeList*: retrieved object.
   */
  //BaseVertexNode* retrieve(Coordinate3D* coord);

  /**
   * Connect the nodes that correspond to triangles sharing a side.
   * @return GenericList<IntPair*>*: list of TriangleNode id pairs for edge creation.
   */
   int createEdges(GenericList<IntPair*>* idList);

   /**
    * Radially ort the TriangleGraphNodes in the edgeList.
    * @return int: 0 on fail, 1 on success.
    */
   int sortEdgeList();

  /**
   * XXX - REMOVE THIS METHOD, DO THE COMPUTATION WITHIN TRIANGLEGRAPH
   * Connect the nodes that correspond to triangles sharing a side.
   * @param graph TriangleGraph*: graph that adds the edges.
   * @return int: 0 on fail, 1 on success.
   */
  //int createEdges2(TriangleGraph* graph);

 private:
  TriangleEdge* edge;
  GenericList<TriangleGraphNode*>* edgeList;
  TriangleEdgeTree* child1;
  TriangleEdgeTree* child2;

};

#endif
